# $Name:  $ 
# $Author: dgp $ 
# $Date: 1997/03/20 17:48:42 $ 
#
# The startup file for the Tcl application Oommf, the Object-Oriented
# MicroMagnetic computational Framework.  Its purpose is to establish
# a suitable Tcl environment for the Oommf application.
#
# In particular, it performs the following tasks
#	o	Makes Oommf's support packages known to the auto-loader
#	o	Checks that the versions of the support packages and the
#		binary are compatible
#	o	Defines the Tcl command 'package' if it is missing (pre-7.5)
#	o	Configures the behavior of the package unknown handler to be
#		consistent with Tcl 7.6 .
#	o	Defines the standard procs and global Tcl variables for
#		the Oommf package.

#bgerror "Starting oommf.tcl!"
# Wrap this entire script in a "catch" to trap its errors.  
# See bottom of file 
if [catch {

# Add the directory named by the OOMMF_PACKAGE_PATH environment variable
# to the path searched by the auto-loader.
#
# This allows users to replace Oommf's support packages with their own 
# replacement implementations.
catch {lappend auto_path $env([string toupper [OMF_PackageName]]_PACKAGE_PATH)}

# Add the directory containing default implementations Oommf's support
# packages to the path searched by the auto_loader.
if {[lsearch -exact $auto_path $omf_library] < 0} {
    lappend auto_path $omf_library
}
#bgerror "Starting package!"
# Check that we can recover from a missing package command, and
# set up for that recovery.
if {[info commands package] == ""} {

# The next line of code is where control over the version of Pkg
# used by this version of Oommf is exercised, instead of by a 
# package require command, in Tcl 7.4.
  proc OMFPKG_Version {} {return 0.0}

# We're actually setting the values of the standard globals of the Package
# package here.  No harm, no foul.
#
  proc OMFPKG_PackageName {} {return Package}
  proc OMFPKG_PackagePrefix {} {return pkg}
  if {![string compare [OMF_FileNameBase] omf]} {
    proc OMFPKG_FileNameBase {} {OMFPKG_PackagePrefix}
  } else {
    proc OMFPKG_FileNameBase {} {string tolower [OMFPKG_PackageName]}
  }
  proc _OMF_CheckPackage {lib ver fnb failmsg} {
    global auto_path
    set pkgDir [file join $lib $fnb$ver]
    set pkgScript [file join $pkgDir $fnb.tcl]
    set pkgIndex [file join $pkgDir tclIndex]
    if {[file exists $pkgDir] && 
      [file isdirectory $pkgDir] && 
      [file exists $pkgScript] &&
      [file exists $pkgIndex] &&
      [file readable $pkgScript] &&
      [file readable $pkgIndex]} {
      if {[lsearch -exact $auto_path $pkgDir] < 0} {
        lappend auto_path $pkgDir
      }
    } else {error $failmsg}
  }
  set _omf_g(msg) "[OMF_PackageName] support requirements are not met.\n\n"
  append _omf_g(msg) "Tcl $tcl_version does not provide "
  append _omf_g(msg) "a \"package\" command.\n\n"
  append _omf_g(msg) "Either re-link [OMF_PackageName] to a Tcl library "
  append _omf_g(msg) "version 7.5 or later, "
  append _omf_g(msg) "or install the Tcl package [OMFPKG_PackageName] "
  append _omg_g(msg) "[OMFPKG_Version] in $omf_library. "
  append _omf_g(msg) "(The latter should have been part of a proper "
  append _omf_g(msg) "[OMF_PackageName] installation.)"
  _OMF_CheckPackage $omf_library [OMFPKG_Version] [OMFPKG_FileNameBase] \
    $_omf_g(msg)
  unset _omf_g(msg)
  rename _OMF_CheckPackage {}
}
#bgerror "Package done!"
# For debugging under Tcl 7.5+ only...
#package require Package 0

# This version of Oommf is written for Tcl/Tk versions 7.4/4.0 or later
#
#	NOTE: In Tcl 7.4, the command 'package' is not part of the 
#		Tcl core.  The attempt to execute it below should
#		invoke the auto-loader, which will load the Pkg package
#		as set up above.
#bgerror "Require Tcl!"
package require Tcl 7.4
#bgerror "Require Tk!"
package require Tk 4.0

# The default package unknown handler in Tcl 7.5 only searches the
# directories in auto_path, not also their subdirectories as in later
# Tcl versions.  The installation layout of Oommf requires the latter
# behavior, so patch it in if necessary here.
#
# If the package unknown has already been replaced to take care of this
# problem as part of the local Tcl installation, this approach will
# still work, but may inefficiently re-read index files already read by
# that other replacement handler.  Might want to develop a way to check
# for this.
#
# An alternate solution would be to add all the sub-directories of 
# directories in auto_path to auto_path, but that approach would not
# dynamically find new packages installed while an interpreter is 
# running.
if {![string compare $tcl_version 7.5]} {
  set _omf_g(defaultPkgUnknown) [package unknown]
  proc _OMF_PkgUnknown {name version {exact {}}} {
    global _omf_g
    $_omf_g(defaultPkgUnknown) $name $version $exact
    global auto_path
    if ![info exists auto_path] {
	return
    }
    for {set i [expr [llength $auto_path] - 1]} {$i >= 0} {incr i -1} {
	set dir [lindex $auto_path $i]
	foreach file [glob -nocomplain [file join $dir * pkgIndex.tcl]] {
	    if [file readable $file] {
		set dir [file dirname $file]
		source $file
	    }
	}
    }
  }
  package unknown _OMF_PkgUnknown
}

# Patch a bug MJD found in the Tk 4.2 distribution
if { $tk_version == 4.2 } {
    # The distribution tkMenuFind source seems to be broken: it has
    # comments in an illegal position inside the 'switch' statement.
    # The following code is from Tk 4.2, but with the comments moved.
    proc tkMenuFind { w char } {
	global tkPriv
	set char [string tolower $char]
	
	foreach child [winfo child $w] {
	    switch [winfo class $child] {
		Menubutton {
		    set char2 [string index [$child cget -text]  \
			    [$child cget -underline]]
		    if {([string compare $char \
			    [string tolower $char2]] == 0) \
			    || ($char == "")} {
			if {[$child cget -state] != "disabled"} {
			    return $child
			}
		    }
		}
		default {
		    # The tag above used to be "Frame", but it was changed so
		    # that the code would work with Itcl 2.0, which apparently
		    # uses other classes of widgets to hold menubuttons.
		    set match [tkMenuFind $child $char]
		    if {$match != ""} {
			return $match
		    }
		}
	    }
	}
	return {}
    }
}
#bgerror "Patches in!"
# We want to use the CvsTools package (defines OMFCVS_PackageProvide)
#bgerror "Require CvsTools!"
package require CvsTools 0

#bgerror "Provide Oommf!"
# Adds Oommf package to database; defines standard global variables
OMFCVS_PackageProvide {^Name: Oommf-0_0a0^} {}

#bgerror "Require HelpBrowser!"
# At this point list additional package requirements of the Oommf application
package require HelpBrowser 0
# Define a few utility routines
proc OMF_FatalError {message} {
  bgerror $msg
  exit
}

proc OMF_InitializationError {msg} {
  set emsg "[OMF_PackageName] initialization error\n\n"
  append emsg $msg
  # For now, make all initialization error fatal
  OMF_FatalError $msg
}

proc OMF_ListCommandInformation {command} {
  puts "\t$command [info args $command]"
}

proc OMF_PublicSummary {} {
  puts "\nSummary of available public commands and global variables"
  puts "========================================================="
  foreach package [package names] {
    set version [package provide $package]
    if {[string compare $version ""]} {
      if {![regexp {^(Tk|Tcl|Dbg|Tcldbg)$} $package]} {
        OMF_PrintPackageInterface $package
      } else {
        puts "$package Package ($version):"
        puts "\tSee documentation"
      }
    } else {
      puts "$package Package:"
      puts "\tNot loaded.  Load with \"package require $package\""
    }
  }
  puts "========================="
}

proc OMF_PrintPackageInterface {package} {
  set p [OMF[string toupper $package]_PackagePrefix]
  set P [string toupper $p]
  puts "$package Package ([OMF${P}_PatchLevel]):"
  puts "    Global variables:"
  foreach variable [info globals omf${p}_*] {
    global $variable
    puts "\t$variable = [set $variable]"
  }
  puts "    Public commands:"
  set cmdlist [info commands OMF[string toupper $package]_*]
  if {[string compare [string toupper $package] $P]} {
    set cmdlist [concat $cmdlist [info commands OMF${P}_*]]
  }
  foreach command $cmdlist {
    OMF_ListCommandInformation $command
  }
}

# Globals should be set now, so define Gui startup
proc _OMF_StartGui {} {
  global omf_library 
  if [catch {source [file join $omf_library [OMF_FileNameBase][OMF_Version] \
                     gui.tcl]} em] {
    set msg "Unable to create user interface\n\n"
    append msg $em
    OMF_InitializationError $msg
  }
}

#bgerror "Procs defined!"
# Close the catch from the top of the file
} _omf_g(em)] { 
# Can't use error message routines -- might not have been defined.
  set _omf_g(msg) "[OMF_PackageName] initialization error\n\n"
  append _omf_g(msg) $_omf_g(em)
  bgerror $_omf_g(msg)
  unset _omf_g(msg)
  unset _omf_g(em)
  exit;
}
